]> git.scripts.mit.edu Git - git.git/commitdiff
pkt-line: teach packet_read_line to chomp newlines
authorJeff King <peff@peff.net>
Wed, 20 Feb 2013 20:02:28 +0000 (15:02 -0500)
committerJunio C Hamano <gitster@pobox.com>
Wed, 20 Feb 2013 21:42:21 +0000 (13:42 -0800)
The packets sent during ref negotiation are all terminated
by newline; even though the code to chomp these newlines is
short, we end up doing it in a lot of places.

This patch teaches packet_read_line to auto-chomp the
trailing newline; this lets us get rid of a lot of inline
chomping code.

As a result, some call-sites which are not reading
line-oriented data (e.g., when reading chunks of packfiles
alongside sideband) transition away from packet_read_line to
the generic packet_read interface. This patch converts all
of the existing callsites.

Since the function signature of packet_read_line does not
change (but its behavior does), there is a possibility of
new callsites being introduced in later commits, silently
introducing an incompatibility.  However, since a later
patch in this series will change the signature, such a
commit would have to be merged directly into this commit,
not to the tip of the series; we can therefore ignore the
issue.

This is an internal cleanup and should produce no change of
behavior in the normal case. However, there is one corner
case to note. Callers of packet_read_line have never been
able to tell the difference between a flush packet ("0000")
and an empty packet ("0004"), as both cause packet_read_line
to return a length of 0. Readers treat them identically,
even though Documentation/technical/protocol-common.txt says
we must not; it also says that implementations should not
send an empty pkt-line.

By stripping out the newline before the result gets to the
caller, we will now treat the newline-only packet ("0005\n")
the same as an empty packet, which in turn gets treated like
a flush packet. In practice this doesn't matter, as neither
empty nor newline-only packets are part of git's protocols
(at least not for the line-oriented bits, and readers who
are not expecting line-oriented packets will be calling
packet_read directly, anyway). But even if we do decide to
care about the distinction later, it is orthogonal to this
patch.  The right place to tighten would be to stop treating
empty packets as flush packets, and this change does not
make doing so any harder.

Signed-off-by: Jeff King <peff@peff.net>
Signed-off-by: Junio C Hamano <gitster@pobox.com>
13 files changed:
builtin/archive.c
builtin/fetch-pack.c
builtin/receive-pack.c
builtin/upload-archive.c
connect.c
daemon.c
fetch-pack.c
pkt-line.c
pkt-line.h
remote-curl.c
send-pack.c
sideband.c
upload-pack.c

index 9a1cfd3dac0123cb65d3db7597221a29e9010bd9..d381ac4147ff6dd3019d223eda2c3256884f2adf 100644 (file)
@@ -56,8 +56,6 @@ static int run_remote_archiver(int argc, const char **argv,
        len = packet_read_line(fd[0], buf, sizeof(buf));
        if (!len)
                die(_("git archive: expected ACK/NAK, got EOF"));
-       if (buf[len-1] == '\n')
-               buf[--len] = 0;
        if (strcmp(buf, "ACK")) {
                if (len > 5 && !prefixcmp(buf, "NACK "))
                        die(_("git archive: NACK %s"), buf + 5);
index 940ae35dc2ce4923a248896eb764f145312d71b4..f73664f433c9b602515fe28456da8c86ef0fe4b8 100644 (file)
@@ -105,8 +105,6 @@ int cmd_fetch_pack(int argc, const char **argv, const char *prefix)
                                int n = packet_read_line(0, line, sizeof(line));
                                if (!n)
                                        break;
-                               if (line[n-1] == '\n')
-                                       n--;
                                string_list_append(&sought, xmemdupz(line, n));
                        }
                }
index 912956378247c4de5c33654cded00b2e13dca491..6679e636c721ec28ac818d5584813dbe8ddea785 100644 (file)
@@ -763,8 +763,6 @@ static struct command *read_head_info(void)
                len = packet_read_line(0, line, sizeof(line));
                if (!len)
                        break;
-               if (line[len-1] == '\n')
-                       line[--len] = 0;
                if (len < 83 ||
                    line[40] != ' ' ||
                    line[81] != ' ' ||
index 1517dec4063126170384e0a6f482c2f89e45122c..d90f0aba44fdac817ac48f4329d72e65f1e8f663 100644 (file)
@@ -40,10 +40,6 @@ int cmd_upload_archive_writer(int argc, const char **argv, const char *prefix)
                if (sent_argv.argc > MAX_ARGS)
                        die("Too many options (>%d)", MAX_ARGS - 1);
 
-               if (buf[len-1] == '\n') {
-                       buf[--len] = 0;
-               }
-
                if (prefixcmp(buf, arg_cmd))
                        die("'argument' token or flush expected");
                argv_array_push(&sent_argv, buf + strlen(arg_cmd));
index 0aa202f885dce3791d6c92bfa434c084f424b145..fe8eb01ae21c15f1013fd398825aeb20f9630336 100644 (file)
--- a/connect.c
+++ b/connect.c
@@ -77,14 +77,13 @@ struct ref **get_remote_heads(int in, struct ref **list,
                int len, name_len;
 
                len = packet_read(in, buffer, sizeof(buffer),
-                                 PACKET_READ_GENTLE_ON_EOF);
+                                 PACKET_READ_GENTLE_ON_EOF |
+                                 PACKET_READ_CHOMP_NEWLINE);
                if (len < 0)
                        die_initial_contact(got_at_least_one_head);
 
                if (!len)
                        break;
-               if (buffer[len-1] == '\n')
-                       buffer[--len] = 0;
 
                if (len > 4 && !prefixcmp(buffer, "ERR "))
                        die("remote error: %s", buffer + 4);
index 4602b46a5c39e1d501143ab4e95b55aff5c8f23b..4f5cd615589901c94c1439c9bff3e870d3d4af32 100644 (file)
--- a/daemon.c
+++ b/daemon.c
@@ -612,7 +612,7 @@ static int execute(void)
                loginfo("Connection from %s:%s", addr, port);
 
        alarm(init_timeout ? init_timeout : timeout);
-       pktlen = packet_read_line(0, line, sizeof(line));
+       pktlen = packet_read(0, line, sizeof(line), 0);
        alarm(0);
 
        len = strlen(line);
index b53a18f923758001e631da0b650dcca6663e6897..f830db224bd6e05534de232331034b204286af5b 100644 (file)
@@ -220,8 +220,6 @@ static enum ack_type get_ack(int fd, unsigned char *result_sha1)
 
        if (!len)
                die("git fetch-pack: expected ACK/NAK, got EOF");
-       if (line[len-1] == '\n')
-               line[--len] = 0;
        if (!strcmp(line, "NAK"))
                return NAK;
        if (!prefixcmp(line, "ACK ")) {
index 8700cf8add94507c67959f1939b9710a7d5cbf52..dc11c407cd8513d423356d16c1b5b0f929264a39 100644 (file)
@@ -164,6 +164,11 @@ int packet_read(int fd, char *buffer, unsigned size, int options)
        ret = safe_read(fd, buffer, len, options);
        if (ret < 0)
                return ret;
+
+       if ((options & PACKET_READ_CHOMP_NEWLINE) &&
+           len && buffer[len-1] == '\n')
+               len--;
+
        buffer[len] = 0;
        packet_trace(buffer, len, 0);
        return len;
@@ -171,7 +176,7 @@ int packet_read(int fd, char *buffer, unsigned size, int options)
 
 int packet_read_line(int fd, char *buffer, unsigned size)
 {
-       return packet_read(fd, buffer, size, 0);
+       return packet_read(fd, buffer, size, PACKET_READ_CHOMP_NEWLINE);
 }
 
 int packet_get_line(struct strbuf *out,
index 8cd326c92237bd5ecc67587e2199c5ea4c1bae67..5d2fb423d60486e8185fbdfef95e75cd31483f00 100644 (file)
@@ -44,11 +44,18 @@ void packet_buf_write(struct strbuf *buf, const char *fmt, ...) __attribute__((f
  * If options does contain PACKET_READ_GENTLE_ON_EOF, we will not die on
  * condition 4 (truncated input), but instead return -1. However, we will still
  * die for the other 3 conditions.
+ *
+ * If options contains PACKET_READ_CHOMP_NEWLINE, a trailing newline (if
+ * present) is removed from the buffer before returning.
  */
 #define PACKET_READ_GENTLE_ON_EOF (1u<<0)
+#define PACKET_READ_CHOMP_NEWLINE (1u<<1)
 int packet_read(int fd, char *buffer, unsigned size, int options);
 
/* Historical convenience wrapper for packet_read that sets no options */
 int packet_read_line(int fd, char *buffer, unsigned size);
 
 int packet_get_line(struct strbuf *out, char **src_buf, size_t *src_len);
index 7be4b53495647ebf06b50bd22a37678481bfcb3a..b28f96504815224b2b1a06d8d07f0c6ef81c6380 100644 (file)
@@ -308,7 +308,7 @@ static size_t rpc_out(void *ptr, size_t eltsize,
 
        if (!avail) {
                rpc->initial_buffer = 0;
-               avail = packet_read_line(rpc->out, rpc->buf, rpc->alloc);
+               avail = packet_read(rpc->out, rpc->buf, rpc->alloc, 0);
                if (!avail)
                        return 0;
                rpc->pos = 0;
@@ -425,7 +425,7 @@ static int post_rpc(struct rpc_state *rpc)
                        break;
                }
 
-               n = packet_read_line(rpc->out, buf, left);
+               n = packet_read(rpc->out, buf, left, 0);
                if (!n)
                        break;
                rpc->len += n;
@@ -579,7 +579,7 @@ static int rpc_service(struct rpc_state *rpc, struct discovery *heads)
        rpc->hdr_accept = strbuf_detach(&buf, NULL);
 
        while (!err) {
-               int n = packet_read_line(rpc->out, rpc->buf, rpc->alloc);
+               int n = packet_read(rpc->out, rpc->buf, rpc->alloc, 0);
                if (!n)
                        break;
                rpc->pos = 0;
index bde796b1bbb80aca5c3e39c0b0365dcbb43b9c09..8c230bf6c925ba6bd3617af62f44cfea8dc09a7b 100644 (file)
@@ -111,10 +111,7 @@ static int receive_status(int in, struct ref *refs)
        int len = packet_read_line(in, line, sizeof(line));
        if (prefixcmp(line, "unpack "))
                return error("did not receive remote status");
-       if (strcmp(line, "unpack ok\n")) {
-               char *p = line + strlen(line) - 1;
-               if (*p == '\n')
-                       *p = '\0';
+       if (strcmp(line, "unpack ok")) {
                error("unpack failed: %s", line + 7);
                ret = -1;
        }
@@ -131,7 +128,6 @@ static int receive_status(int in, struct ref *refs)
                        break;
                }
 
-               line[strlen(line)-1] = '\0';
                refname = line + 3;
                msg = strchr(refname, ' ');
                if (msg)
index 8f7b25bf7915ec80d55d8a9f6fa6a8953b495204..15cc1aec22115a9f0d41d9e06e29184f64f1a86a 100644 (file)
@@ -38,7 +38,7 @@ int recv_sideband(const char *me, int in_stream, int out)
 
        while (1) {
                int band, len;
-               len = packet_read_line(in_stream, buf + pf, LARGE_PACKET_MAX);
+               len = packet_read(in_stream, buf + pf, LARGE_PACKET_MAX, 0);
                if (len == 0)
                        break;
                if (len < 1) {
index afc2d9279cdcd57b235cbf69590ffc102f2397b5..6e6d16687640e561cbaa745a2d851a83f3630681 100644 (file)
@@ -50,13 +50,6 @@ static void reset_timeout(void)
        alarm(timeout);
 }
 
-static int strip(char *line, int len)
-{
-       if (len && line[len-1] == '\n')
-               line[--len] = 0;
-       return len;
-}
-
 static ssize_t send_client_data(int fd, const char *data, ssize_t sz)
 {
        if (use_sideband)
@@ -447,7 +440,6 @@ static int get_common_commits(void)
                        got_other = 0;
                        continue;
                }
-               strip(line, len);
                if (!prefixcmp(line, "have ")) {
                        switch (got_sha1(line+5, sha1)) {
                        case -1: /* they have what we do not */
@@ -44,11 +44,18 @@ void packet_buf_write(struct strbuf *buf, const char *fmt, ...) __attribute__((f
 int packet_read(int fd, char *buffer, unsigned size, int options);
 
-/* Historical convenience wrapper for packet_read that sets no options */
+/*
+ * Convenience wrapper for packet_read that is not gentle, and sets the
+ * CHOMP_NEWLINE option.
+ */
 int packet_read_line(int fd, char *buffer, unsigned size);
 